@@ -32,6 +32,20 @@ def sql_call(*, use_iterator=False):
     return list(qs)
 
 
+async def async_sql_call(*, use_iterator=False):
+    qs = User.objects.all()
+    if use_iterator:
+        qs = qs.iterator()
+    return await sync_to_async(list)(qs)
+
+
+async def concurrent_async_sql_call(*, use_iterator=False):
+    qs = User.objects.all()
+    if use_iterator:
+        qs = qs.iterator()
+    return await asyncio.gather(sync_to_async(list)(qs), User.objects.acount())
+
+
 class SQLPanelTestCase(BaseTestCase):
     panel_id = "SQLPanel"
 
@@ -57,6 +71,38 @@ class SQLPanelTestCase(BaseTestCase):
         # ensure the stacktrace is populated
         self.assertTrue(len(query["stacktrace"]) > 0)
 
+    async def test_recording_async(self):
+        self.assertEqual(len(self.panel._queries), 0)
+
+        await async_sql_call()
+
+        # ensure query was logged
+        self.assertEqual(len(self.panel._queries), 1)
+        query = self.panel._queries[0]
+        self.assertEqual(query["alias"], "default")
+        self.assertTrue("sql" in query)
+        self.assertTrue("duration" in query)
+        self.assertTrue("stacktrace" in query)
+
+        # ensure the stacktrace is populated
+        self.assertTrue(len(query["stacktrace"]) > 0)
+
+    async def test_recording_concurrent_async(self):
+        self.assertEqual(len(self.panel._queries), 0)
+
+        await concurrent_async_sql_call()
+
+        # ensure query was logged
+        self.assertEqual(len(self.panel._queries), 2)
+        query = self.panel._queries[0]
+        self.assertEqual(query["alias"], "default")
+        self.assertTrue("sql" in query)
+        self.assertTrue("duration" in query)
+        self.assertTrue("stacktrace" in query)
+
+        # ensure the stacktrace is populated
+        self.assertTrue(len(query["stacktrace"]) > 0)
+
     @unittest.skipUnless(
         connection.vendor == "postgresql", "Test valid only on PostgreSQL"
     )
@@ -609,8 +655,8 @@ class SQLPanelTestCase(BaseTestCase):
         template_info = query["template_info"]
         template_name = os.path.basename(template_info["name"])
         self.assertEqual(template_name, "flat.html")
-        self.assertEqual(template_info["context"][2]["content"].strip(), "{{ users }}")
-        self.assertEqual(template_info["context"][2]["highlight"], True)
+        self.assertEqual(template_info["context"][3]["content"].strip(), "{{ users }}")
+        self.assertEqual(template_info["context"][3]["highlight"], True)
 
     @override_settings(
         DEBUG=True,
@@ -683,6 +729,13 @@ class SQLPanelTestCase(BaseTestCase):
         self.assertNotEqual(queries[0]["similar_color"], queries[3]["similar_color"])
         self.assertNotEqual(queries[0]["duplicate_color"], queries[3]["similar_color"])
 
+    def test_explain_with_union(self):
+        list(User.objects.filter(id__lt=20).union(User.objects.filter(id__gt=10)))
+        response = self.panel.process_request(self.request)
+        self.panel.generate_stats(self.request, response)
+        query = self.panel._queries[0]
+        self.assertTrue(query["is_select"])
+
 
 class SQLPanelMultiDBTestCase(BaseMultiDBTestCase):
     panel_id = "SQLPanel"
